@@ -1,21 +1,7 @@
 /**
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "lanlinkprovider.h"
@@ -37,6 +23,7 @@
 #include <QNetworkConfigurationManager>
 #include <QSslCipher>
 #include <QSslConfiguration>
+#include <QSslKey>
 
 #include "daemon.h"
 #include "landevicelink.h"
@@ -53,7 +40,7 @@ LanLinkProvider::LanLinkProvider(
         bool testMode,
         quint16 udpBroadcastPort,
         quint16 udpListenPort
-    )
+        )
     : m_server(new Server(this))
     , m_udpSocket(this)
     , m_tcpPort(0)
@@ -102,10 +89,10 @@ void LanLinkProvider::onStart()
         // Refer to https://doc.qt.io/qt-5/qabstractsocket.html#SocketError-enum to decode socket error number
         QString errorMessage = QString::fromLatin1(QMetaEnum::fromType<QAbstractSocket::SocketError>().valueToKey(sockErr));
         qCritical(KDECONNECT_CORE)
-            << QLatin1String("Failed to bind UDP socket on port")
-            << m_udpListenPort
-            << QLatin1String("with error")
-            << errorMessage;
+                << QLatin1String("Failed to bind UDP socket on port")
+                << m_udpListenPort
+                << QLatin1String("with error")
+                << errorMessage;
     }
     Q_ASSERT(success);
 
@@ -133,7 +120,7 @@ void LanLinkProvider::onStop()
 void LanLinkProvider::onNetworkChange()
 {
     if (m_combineBroadcastsTimer.isActive()) {
-        qCDebug(KDECONNECT_CORE()) << "Preventing duplicate broadcasts";
+        qCDebug(KDECONNECT_CORE) << "Preventing duplicate broadcasts";
         return;
     }
     m_combineBroadcastsTimer.start();
@@ -142,7 +129,6 @@ void LanLinkProvider::onNetworkChange()
 //I'm in a new network, let's be polite and introduce myself
 void LanLinkProvider::broadcastToNetwork()
 {
-
     if (!m_server->isListening()) {
         //Not started
         return;
@@ -152,9 +138,9 @@ void LanLinkProvider::broadcastToNetwork()
 
     qCDebug(KDECONNECT_CORE()) << "Broadcasting identity packet";
 
-    QHostAddress destAddress = m_testMode? QHostAddress::LocalHost : QHostAddress(QStringLiteral("255.255.255.255"));
+    QList<QHostAddress> destinations = getBroadcastAddresses();
 
-    NetworkPacket np(QLatin1String(""));
+    NetworkPacket np;
     NetworkPacket::createIdentityPacket(&np);
     np.set(QStringLiteral("tcpPort"), m_tcpPort);
 
@@ -164,22 +150,54 @@ void LanLinkProvider::broadcastToNetwork()
     sendSocket.setProxy(QNetworkProxy::NoProxy);
     for (const QNetworkInterface& iface : QNetworkInterface::allInterfaces()) {
         if ( (iface.flags() & QNetworkInterface::IsUp)
-          && (iface.flags() & QNetworkInterface::IsRunning)
-          && (iface.flags() & QNetworkInterface::CanBroadcast)) {
+             && (iface.flags() & QNetworkInterface::IsRunning)
+             && (iface.flags() & QNetworkInterface::CanBroadcast)) {
             for (const QNetworkAddressEntry& ifaceAddress : iface.addressEntries()) {
                 QHostAddress sourceAddress = ifaceAddress.ip();
                 if (sourceAddress.protocol() == QAbstractSocket::IPv4Protocol && sourceAddress != QHostAddress::LocalHost) {
                     qCDebug(KDECONNECT_CORE()) << "Broadcasting as" << sourceAddress;
-                    sendSocket.writeDatagram(np.serialize(), destAddress, m_udpBroadcastPort);
+                    sendBroadcasts(sendSocket, np, destinations);
                     sendSocket.close();
                 }
             }
         }
     }
 #else
-    m_udpSocket.writeDatagram(np.serialize(), destAddress, m_udpBroadcastPort);
+    sendBroadcasts(m_udpSocket, np, destinations);
 #endif
+}
 
+QList<QHostAddress> LanLinkProvider::getBroadcastAddresses()
+{
+    const QStringList customDevices = KdeConnectConfig::instance().customDevices();
+
+    QList<QHostAddress> destinations;
+    destinations.reserve(customDevices.length() + 1);
+
+    // Default broadcast address
+    destinations.append(m_testMode ? QHostAddress::LocalHost : QHostAddress::Broadcast);
+
+    // Custom device addresses
+    for (auto& customDevice : customDevices) {
+        QHostAddress address(customDevice);
+        if (address.isNull()) {
+            qCWarning(KDECONNECT_CORE) << "Invalid custom device address" << customDevice;
+        } else {
+            destinations.append(address);
+        }
+    }
+
+    return destinations;
+}
+
+void LanLinkProvider::sendBroadcasts(
+        QUdpSocket& socket, const NetworkPacket& np, const QList<QHostAddress>& addresses)
+{
+    const QByteArray payload = np.serialize();
+
+    for (auto& address : addresses) {
+        socket.writeDatagram(payload, address, m_udpBroadcastPort);
+    }
 }
 
 //I'm the existing device, a new device is kindly introducing itself.
@@ -518,19 +536,26 @@ void LanLinkProvider::configureSslSocket(QSslSocket* socket, const QString& devi
     // Configure for ssl
     QSslConfiguration sslConfig;
     sslConfig.setCiphers(socketCiphers);
+    sslConfig.setLocalCertificate(KdeConnectConfig::instance().certificate());
 
-    socket->setSslConfiguration(sslConfig);
-    socket->setLocalCertificate(KdeConnectConfig::instance().certificate());
-    socket->setPrivateKey(KdeConnectConfig::instance().privateKeyPath());
-    socket->setPeerVerifyName(deviceId);
+    QFile privateKeyFile(KdeConnectConfig::instance().privateKeyPath());
+    QSslKey privateKey;
+    if (privateKeyFile.open(QIODevice::ReadOnly)) {
+        privateKey = QSslKey(privateKeyFile.readAll(), QSsl::Rsa);
+    }
+    privateKeyFile.close();
+    sslConfig.setPrivateKey(privateKey);
 
     if (isDeviceTrusted) {
         QString certString = KdeConnectConfig::instance().getDeviceProperty(deviceId, QStringLiteral("certificate"), QString());
-        socket->addCaCertificate(QSslCertificate(certString.toLatin1()));
-        socket->setPeerVerifyMode(QSslSocket::VerifyPeer);
+        sslConfig.setCaCertificates({QSslCertificate(certString.toLatin1())});
+        sslConfig.setPeerVerifyMode(QSslSocket::VerifyPeer);
     } else {
-        socket->setPeerVerifyMode(QSslSocket::QueryPeer);
+        sslConfig.setPeerVerifyMode(QSslSocket::QueryPeer);
     }
+    socket->setSslConfiguration(sslConfig);
+    socket->setPeerVerifyName(deviceId);
+
 
     //Usually SSL errors are only bad for trusted devices. Uncomment this section to log errors in any case, for debugging.
     //QObject::connect(socket, static_cast<void (QSslSocket::*)(const QList<QSslError>&)>(&QSslSocket::sslErrors), [](const QList<QSslError>& errors)
@@ -547,23 +572,23 @@ void LanLinkProvider::configureSocket(QSslSocket* socket) {
 
     socket->setSocketOption(QAbstractSocket::KeepAliveOption, QVariant(1));
 
-    #ifdef TCP_KEEPIDLE
-        // time to start sending keepalive packets (seconds)
-        int maxIdle = 10;
-        setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPIDLE, &maxIdle, sizeof(maxIdle));
-    #endif
+#ifdef TCP_KEEPIDLE
+    // time to start sending keepalive packets (seconds)
+    int maxIdle = 10;
+    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPIDLE, &maxIdle, sizeof(maxIdle));
+#endif
 
-    #ifdef TCP_KEEPINTVL
-        // interval between keepalive packets after the initial period (seconds)
-        int interval = 5;
-        setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPINTVL, &interval, sizeof(interval));
-    #endif
+#ifdef TCP_KEEPINTVL
+    // interval between keepalive packets after the initial period (seconds)
+    int interval = 5;
+    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPINTVL, &interval, sizeof(interval));
+#endif
 
-    #ifdef TCP_KEEPCNT
-        // number of missed keepalive packets before disconnecting
-        int count = 3;
-        setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPCNT, &count, sizeof(count));
-    #endif
+#ifdef TCP_KEEPCNT
+    // number of missed keepalive packets before disconnecting
+    int count = 3;
+    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPCNT, &count, sizeof(count));
+#endif
 
 }
 
