@@ -1,21 +1,7 @@
 /**
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "shareplugin.h"
@@ -33,7 +19,7 @@
 #include <KPluginFactory>
 #include <KIO/Job>
 #include <KIO/MkpathJob>
-#include <KMimeTypeTrader>
+#include <KApplicationTrader>
 #include <KFileUtils>
 
 #include "core/filetransferjob.h"
@@ -51,7 +37,7 @@ SharePlugin::SharePlugin(QObject* parent, const QVariantList& args)
 QUrl SharePlugin::destinationDir() const
 {
     const QString defaultDownloadPath = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
-    QUrl dir = QUrl::fromLocalFile(config()->get<QString>(QStringLiteral("incoming_path"), defaultDownloadPath));
+    QUrl dir = QUrl::fromLocalFile(config()->getString(QStringLiteral("incoming_path"), defaultDownloadPath));
 
     if (dir.path().contains(QLatin1String("%1"))) {
         dir.setPath(dir.path().arg(device()->name()));
@@ -123,9 +109,10 @@ bool SharePlugin::receivePacket(const NetworkPacket& np)
         if (np.hasPayload()) {
             qint64 dateModified = np.get<qint64>(QStringLiteral("lastModified"), QDateTime::currentMSecsSinceEpoch());
             const bool open = np.get<bool>(QStringLiteral("open"), false);
-            
+
             if (!m_compositeJob) {
                 m_compositeJob = new CompositeFileTransferJob(device()->id());
+                m_compositeJob->setProperty("destUrl", destinationDir().toString());
                 KIO::getJobTracker()->registerJob(m_compositeJob);
             }
 
@@ -145,7 +132,7 @@ bool SharePlugin::receivePacket(const NetworkPacket& np)
     } else if (np.has(QStringLiteral("text"))) {
         QString text = np.get<QString>(QStringLiteral("text"));
 
-        KService::Ptr service = KMimeTypeTrader::self()->preferredService(QStringLiteral("text/plain"));
+        KService::Ptr service = KApplicationTrader::preferredService(QStringLiteral("text/plain"));
         const QString defaultApp = service ? service->desktopEntryName() : QString();
 
         if (defaultApp == QLatin1String("org.kde.kate") || defaultApp == QLatin1String("org.kde.kwrite")) {
